from setuptools import setup

setup(name='mydbs',
      version='0.1.2',
      description='databases operations without sqls',
      url= 'https://mehralsaugen.github.io/',
      author='shockwave',
      author_email='yao_yingzhe@163.com',
      license='MIT',
      packages=['mydbs'])